import * as React from 'react';
import {
    TouchableHighlight,
    TouchableNativeFeedback,
    TouchableOpacity,
    TouchableWithoutFeedback
} from 'react-native-gesture-handler';
import {Keyboard, StyleProp, View, ViewProps} from 'react-native';
import {debounce} from 'lodash';

const buttonComponents: {
    [key: string]: any
} = {
    'opacity': TouchableOpacity,
    'withoutFeedback': TouchableWithoutFeedback,
    'nativeFeedback': TouchableNativeFeedback,
    'highlight': TouchableHighlight
}

function getButtonComponent(type: string): any {
    return buttonComponents[type];
}

export type ButtonProps = {
    buttonType?: 'opacity' | 'withoutFeedback' | 'nativeFeedback' | 'highlight',
    onPress?: () => void;
    disabled?: boolean;
    style?: StyleProp<ViewProps>;
    children?: React.ReactNode;
}

export default function Button(props: ButtonProps) {
    const {
        buttonType = 'nativeFeedback',
        onPress = () => {
        },
        disabled,
        style,
        children,
        ...otherProps
    } = props;

    const ButtonComponent = getButtonComponent(buttonType);
    const [processing, setProcessing] = React.useState(false);

    const _onPress = React.useCallback(debounce(async () => {
        Keyboard.dismiss();

        if (disabled || processing) {
            return;
        }

        setProcessing(true);

        if (onPress) {
            await onPress();
        }

        setProcessing(false);
    }, 500, {leading: true, trailing: false}), [onPress, disabled, processing]);

    return (
        <ButtonComponent
            {...otherProps}
            onPress={_onPress}
            disabled={disabled || processing}>
            <View style={style}>
                {children}
            </View>
        </ButtonComponent>
    );
}